"""
Manage your Anaconda Server server-side settings

###### Set server-side settings

You can add a site named **site_name** like this:

    conda repo admin --set SETTING_NAME SETTING_VALUE

Currently supported settings are:

    - `user_channel_autocreate` - if True, a user's channel is created automatically on first login

###### Show server-side settings

Yuo can see current settings:

    conda repo admin --show
"""
import logging
from argparse import RawDescriptionHelpFormatter

from .. import errors
from ..utils.format import SettingsFormatter
from ..utils.yaml import safe_load
from .base import SubCommandBase

logger = logging.getLogger("repo_cli")


SETTINGS_VALIDATOR = {"user_channel_autocreate": safe_load}


class SubCommand(SubCommandBase):
    name = "admin"

    def main(self):

        if self.args.show:
            self.show_settings()
            return

        if self.args.get:
            self.show_settings(self.args.get)
            return

        if self.args.set:
            self.update_settings(self.args.set)
            return

        raise NotImplementedError("Please use command options")

    def show_settings(self, key=None):
        settings = self.api.get_system_settings()
        if key is not None:
            if key not in settings:
                raise errors.RepoCLIError("%s is an unknown admin setting" % key)
            settings = {key: settings[key]}

        self.log.info(SettingsFormatter.format_object_as_list(settings))

    def update_settings(self, args):
        data = {}
        for key, value in args:
            if key not in SETTINGS_VALIDATOR:
                raise errors.RepoCLIError("%s is an unknown admin setting" % key)
            data[key] = SETTINGS_VALIDATOR[key](value)

        settings = self.api.get_system_settings()
        settings.update(data)
        self.api.update_system_settings(settings)

        self.log.info("Anaconda Server settings are updated")

    def add_parser(self, subparsers):
        description = "Anaconda Server admin settings"
        parser = subparsers.add_parser(
            "admin",
            help=description,
            description=description,
            epilog=__doc__,
            formatter_class=RawDescriptionHelpFormatter,
        )

        agroup = parser.add_argument_group("actions")

        agroup.add_argument(
            "--set",
            nargs=2,
            action="append",
            default=[],
            help="sets a server setting value: name value",
            metavar=("name", "value"),
        )
        agroup.add_argument("--get", metavar="name", help="get value: name")
        agroup.add_argument(
            "--show", action="store_true", default=False, help="show all variables"
        )

        parser.set_defaults(main=self.main, sub_parser=parser)



"""
Manage your Anaconda repository channels.
"""

from __future__ import print_function, unicode_literals

import argparse

from ..utils.format import CVEFilesFormatter, CVEFormatter
from .base import SubCommandBase


class SubCommand(SubCommandBase):
    name = "cves"

    def main(self):
        self.log.info("")
        args = self.args
        if args.list:
            self.show_list(args.offset, args.limit)
        elif args.show:
            self.show(args.show)
        elif args.show_files:
            self.show_files(args.show_files, args.offset, args.limit)
        else:
            raise NotImplementedError()

    def show_list(self, offset=0, limit=50):
        data = self.api.get_cves(offset, limit)
        self.log.info(CVEFormatter.format_list(data["items"]))
        self.log.info("")

    def show(self, cve):
        data = self.api.get_cve(cve)
        self.log.info(CVEFormatter.format_detail(data))
        self.log.info("")

    def show_files(self, cve_id, offset=0, limit=50):
        data = self.api.get_cve_files(cve_id, offset, limit)
        self.log.info(
            "Showing %d packages, associated with %s, starting from %d from total %d:"
            % (limit, cve_id, offset, data["total_count"])
        )
        self.log.info("")
        self.log.info(CVEFilesFormatter.format_list(data["items"]))
        self.log.info("")

    def add_parser(self, subparsers):
        subparser = subparsers.add_parser(
            self.name,
            help="Access Anaconda Repository {}s".format(self.name),
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=__doc__,
        )

        group = subparser.add_mutually_exclusive_group(required=True)

        group.add_argument(
            "--list",
            action="store_true",
            help="list all {}s for a user".format(self.name),
        )
        group.add_argument(
            "--show",
            metavar=self.name.upper(),
            help="Show details about {}".format(self.name),
        )
        group.add_argument(
            "--show-files",
            metavar=self.name.upper(),
            help="Show files for {} (Use limit/offset additionally)".format(self.name),
        )

        subparser.add_argument(
            "-o",
            "--offset",
            default=0,
            type=int,
            help="Offset when displaying the results",
        )
        subparser.add_argument(
            "-l",
            "--limit",
            default=50,
            type=int,
            help="Offset when displaying the results",
        )

        subparser.set_defaults(main=self.main)
